/******************************************************************************
 * debug.c
 *
 * Generic debug routines for the gdbproxy and SOL-connect
 *
 * Copyright (C) 2006 Tony Breeds, IBM Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation; or, when distributed
 * separately from the Linux kernel or incorporated into other
 * software packages, subject to the following license:
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this source file (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <err.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "debug.h"

#ifdef DEBUG

static int debug_fd;

void debug_init(const char *file_name)
{
	int len;
	char buf[4096];

	debug_fd = open(file_name, O_CREAT|O_TRUNC|O_SYNC|O_WRONLY, 
	                S_IRUSR|S_IWUSR);
	if (debug_fd < 0)
		err(1, "Couldn't opne debug logfile");
	/* FIXME: Add timestamp */
	len = sprintf(buf, "Log opened\n");
	write(debug_fd, buf, len);
}

void debug_log_data(char *msg, int fd, uint8_t *buf, int buf_len)
{
	int len;
	char header[4096];

	/* Skip bogus data */
	if (buf_len == 1 && buf[0] == '\x0')
		return;

	len = sprintf(header, "\n%s(%d) %d bytes : ", msg, fd, buf_len);
	write(debug_fd, header, len);
	write(debug_fd, buf, buf_len);
	fdatasync(debug_fd);
}

#endif
