/******************************************************************************
 * exec.c
 *
 * Spawn a program for bi-directional comms, both gdbproxy and SOL-connect need
 * to do this.
 *
 * Copyright (C) 2006 Tony Breeds, IBM Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation; or, when distributed
 * separately from the Linux kernel or incorporated into other
 * software packages, subject to the following license:
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this source file (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <err.h>
#include <sys/types.h>
#include <sys/time.h>

#include "exec.h"
#include "debug.h"

#define BASH_PATH	"/bin/bash"

int run_cmd(const char *cmd, int *to_cmd, int *from_cmd)
{
	pid_t pid;
	size_t len;
	char buf[4096];
	int pin[2], pout[2];

	len = sprintf(buf, "%s %s", "exec", cmd);

	if (pipe(pin) < 0 || pipe(pout) < 0)
		err(1, "Creating pipes failed");

	debug("Executing proxy command: %.500s", buf);
	pid = fork();
	if (pid == 0) {
		char *argv[4];

		/* Redirect stdin and stdout. */
		close(pin[1]);
		if (pin[0] != 0) {
			if (dup2(pin[0], 0) < 0)
				warn("dup2 stdin");
			close(pin[0]);
		}
		close(pout[0]);
		if (dup2(pout[1], 1) < 0)
			warn("dup2 stdout");
		close(pout[1]);

		/* Leave stderr alone */
		argv[0] = BASH_PATH;
		argv[1] = "-c";
		argv[2] = buf;
		argv[3] = NULL;

		execv(argv[0], argv);
		return -1;
	}

	/* Parent. */
	if (pid < 0)
		err(1, "fork() barfed!");

	/* Close child side of the descriptors. */
	close(pin[0]);
	close(pout[1]);

	*from_cmd = pout[0];
	*to_cmd = pin[1];

	return pid;
}
